module Adparlor
  module Facebook
    module GraphApi
      class AdAccount < GraphObject
        include Fields::AdAccount
        include Traits::Methods
        field_attrs FIELDS
        allow_method :create, :destroy, :update

        class << self
          def get(path_or_account_id, options = {})
            path = path_or_account_id.to_s
            if !path.include?('adaccounts')
              begin
                account_id = Integer(path_or_account_id)
                path = "/#{normalize_account_id(account_id)}"
              rescue
                STDERR.puts 'Invalid account id passed to get'
              end
            end

            super(path, options)
          end

          def normalize_account_id(account_id)
            account_id = account_id.to_s.downcase
            if account_id.include?('act_')
              account_id = account_id
            else
              account_id = "act_#{account_id}"
            end
            account_id
          end

          def parse_data_for_collection(response)
            if !response['data'].first.is_a?(Hash)
              response['data'].first.last
            else
              response['data']
            end
          end
        end

        def normalized_account_id(account_id)
          "act_#{account_id}"
        end

        def activities
          @activities ||= CollectionProxy.new Activity, endpoint('activities'), access_token
        end

        def adcreatives
          @adcreatives ||= CollectionProxy.new AdCreative, endpoint('adcreatives'), access_token
        end

        def adimages
          @adimages ||= CollectionProxy.new AdImage, endpoint('adimages'), access_token
        end

        def adlabels
          @adlabels ||= CollectionProxy.new AdLabel, endpoint('adlabels'), access_token
        end

        def ads
          @ads ||= CollectionProxy.new Ad, endpoint('ads'), access_token
        end

        def adsets
          @adsets ||= CollectionProxy.new AdSet, endpoint('adsets'), access_token
        end

        def adspixels
          @adspixels ||= CollectionProxy.new AdsPixel, endpoint('adspixels'), access_token
        end

        def advertisable_applications
          @advertisable_applications ||= CollectionProxy.new AdvertisableApplication, endpoint('advertisable_applications'), access_token
        end

        def advideos
          @advideos ||= CollectionProxy.new AdVideo, endpoint('advideos'), access_token
        end

        def applications
          @applications ||= CollectionProxy.new Application, endpoint('applications'), access_token
        end

        def broadtargetingcategories
          @broadtargetingcategories ||= CollectionProxy.new BroadTargetingCategory, endpoint('broadtargetingcategories'), access_token
        end

        # Deprecated in v2.6. Please Use advertisable_applications, pages, places endpoint instead
        def connectionobjects
          Kernel::fail("connectionobjects is deprecated in v2.6. Use advertisable_applications, pages, places endpoint instead")
        end

        def campaigns
          @campaigns ||= CollectionProxy.new Campaign, endpoint('campaigns'), access_token
        end

        def customaudiences
          @customaudiences ||= CollectionProxy.new CustomAudience, endpoint('customaudiences'), access_token
        end

        def customaudiencestos
          @customaudiencestos ||= CollectionProxy.new CustomAudienceTos, endpoint('customaudiencestos'), access_token
        end

        def customconversions
          @customconversions ||= CollectionProxy.new CustomConversion, endpoint('customconversions'), access_token
        end

        def delivery_estimate
          @delivery_estimate ||= CollectionProxy.new DeliveryEstimate, endpoint('delivery_estimate'), access_token
        end

        def generatepreviews
          @generatepreviews ||= CollectionProxy.new AdPreview, endpoint('generatepreviews'), access_token
        end

        def insights
          @insights ||= CollectionProxy.new Insight, endpoint('insights'), access_token
        end

        def instagram_accounts
          @instagram_accounts ||= CollectionProxy.new InstagramAccount, endpoint('instagram_accounts'), access_token
        end

        def offsitepixels
          Kernel::fail("offsite_pixels is deprecated in v2.7 and above.")
        end

        def pages
          @pages ||= CollectionProxy.new PageAndPlace, 'me/accounts', access_token
        end

        def places
          @places ||= CollectionProxy.new PageAndPlace, 'me/accounts', access_token, nil, is_place: true
        end

        def partnercategories
          @partnercategories ||= CollectionProxy.new PartnerCategory, endpoint('partnercategories'), access_token
        end

        def reachestimate
          @reachestimate ||= CollectionProxy.new ReachEstimate, endpoint('reachestimate'), access_token
        end

        def reachfrequencypredictions
          @reachfrequencypredictions ||= CollectionProxy.new ReachFrequencyPrediction, endpoint('reachfrequencypredictions'), access_token
        end

        def targetingsentencelines
          @targetingsentencelines ||= CollectionProxy.new TargetingSentenceLine, endpoint('targetingsentencelines'), access_token
        end

        def targetingsuggestions
          @targetingsuggestions ||= CollectionProxy.new TargetingSuggestion, endpoint('targetingsuggestions'), access_token
        end

        def targetingvalidation
          @targetingvalidation ||= CollectionProxy.new TargetingValidation, endpoint('targetingvalidation'), access_token
        end

        private

        def endpoint(connection)
          "/#{normalized_account_id(account_id)}/#{connection}"
        end
      end
    end
  end
end
